import itertools

FILTERS = ('my_oct', 'my_hex', 'my_bin')
METHODS = ('LAMBDA', 'LAMBDA2', 'DEF', 'MIMIC', 'STATIC', 'BUILTIN')


class Mimic(object):
    """Mimics class I want need to use.
    Initialization take an argument and uses it in method.
    """
    def __init__(self, func):
        self.func = func

    def method(self, v):
        return self.func(v)


class MyFilters(object):
    """Just using the class as a namespace."""
    @staticmethod
    def my_oct(x):
        return oct(x)

    @staticmethod
    def my_hex(x):
        return hex(x)

    @staticmethod
    def my_bin(x):
        return bin(x)


def make_filters(filter_names, method):
    if method == 'LAMBDA':
        # Give me lambdas to the built in functions
        filters = {_: lambda x, f=__builtins__[ _[3:]]: f(x)
                   for _ in filter_names}
    elif method == 'LAMBDA2':
        filters = {_: (lambda x, f=_[3:]: __builtins__[f](x))
                   for _ in filter_names}
    elif method == 'DEF':
        filters = {}
        for name in filter_names:
            def a_filter(x, f=__builtins__[name[3:]]):
                return f(x)
            filters[name] = a_filter
    elif method == 'BUILTIN':
        filters = {name: __builtins__[name[3:]] for name in filter_names}
    elif method == 'STATIC':
        filters = {name: getattr(MyFilters, name) for name in filter_names}
    elif method == 'MIMIC':
        filters = {name: (lambda x, f=Mimic(__builtins__[name[3:]]): f.method(x))
                   for name in filter_names}
    return filters



import itertools
import pytest
import helper
from helper import FILTERS, METHODS, make_filters


@pytest.mark.parametrize('filter_,method',
                         itertools.product(FILTERS, METHODS))
def test_all_filters(filter_, method):
    filters = make_filters(FILTERS, method)
    func = __builtins__[filter_[3:]]
    assert func(254) == filters[filter_](254)



"""
Import code modules
I always forget how to do this.  I finally nailed it down.
Below is my source of information.
https://www.kennethreitz.org/essays/repository-structure-and-python
"""
import os
import sys
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
import flask_obscure as obscure



import pytest
from flask import Flask
import context
from flask_obscure import Obscure
from test_filter import B


def make_app(salt=None):
    app = Flask(__name__)

    if salt is not None:
        app.config["OBSCURE_SALT"] = salt
    return app


def test_salt_from_config():
    app = make_app(0x1234)
    obscure = Obscure(app)

    assert obscure.salt == 0x1234


def test_salt_init_over_config():
    # Parameter salt superior to config
    app = make_app(0x54321)
    obscure = Obscure(app, 0x1234)

    assert obscure.salt == 0x1234


def test_salt_initapp_over_config():
    app = make_app(0x54321)
    obscure = Obscure()
    obscure.init_app(app, 0x1234)

    assert obscure.salt == 0x1234


def test_obscure_init_salted():
    app = make_app()
    obscure = Obscure(salt=0x54321)
    obscure.init_app(app)

    assert 0x54321 == obscure.salt


def test_missing_salt():
    app = make_app()
    with pytest.raises(KeyError):
        Obscure(app).transform(0)


def test_multi_apps():
    app1 = make_app()
    app2 = make_app()

    salt = 0x12345
    obscure = Obscure()
    obscure.init_app(app1, salt)
    obscure.init_app(app2, salt)

    @app1.route("/<tame:number>")
    def index1(number):
        return "\n".join(("#1", str(number)))

    @app2.route("/<tame:number>")
    def index2(number):
        return "\n".join(("#2", str(number)))

    tame = obscure.encode_tame(0)
    with app1.test_client() as go:
        rv = go.get("/" + tame)
        assert 200 == rv.status_code
        assert rv.data.startswith(B("#1\n"))
        assert rv.data.endswith(B("\n0"))

    with app2.test_client() as go:
        rv = go.get("/" + tame)
        assert 200 == rv.status_code
        assert rv.data.startswith(B("#2\n"))
        assert rv.data.endswith(B("\n0"))



import sys
import pytest
from flask import Flask, url_for, request, render_template_string
from jinja2 import Environment
import context
from obscure import Obscure
import flask_obscure as obscure

SALT = 0x1234
FILTERS = tuple(obscure.converters.keys())
TRANSLATE = {
    "num": "transform",
    "hex": "encode_hex",
    "b32": "encode_base32",
    "b64": "encode_base64",
    "tame": "encode_tame",
}


def B(s):
    # Yes, a proper if/else statement would be more readable, but I am
    # doing this for coverage.  Tox would also get this.
    return s if sys.version_info.major == 2 else bytes(s, "ascii")


def make_jinja_filter_for(encoder):
    enc = TRANSLATE[encoder]
    return lambda x, s=SALT, f=enc: str(getattr(Obscure(s), f)(x))


@pytest.fixture(scope="function")
def app():
    _app = Flask(__name__)
    obscure.Obscure(_app, SALT)  # works for test, but use flask.config

    def invoice(customer_id):
        """Show obfuscated customer number.
        Routes for each filter are created like:
            /invoice/b32/<b32:customer_id>
            /invoice/b64/<b64:customer_id>
            /invoice/hex/<hex:customer_id>
            /invoice/tame/<tame:customer_id>

        Arg:
            customer_id: actual customer number

            This number, while obfuscated in the URL, has been converted
            by the in the url_map.converters at this point.

        Return string:
            {obscure method}#{obscured customerID}
        """
        encoder = request.url_rule.endpoint
        template = "{{ encoder }}#{{ customer_id|%s }}" % encoder
        return render_template_string(template, **locals())

    for f in FILTERS:
        url = "/invoice/%s/<%s:customer_id>" % (f, f)
        invoice = _app.route(url, endpoint=f)(invoice)
    return _app


@pytest.mark.parametrize("encoder", FILTERS)
def test_flask_filter(app, encoder):
    obs = app.url_map.converters[encoder](SALT)
    with app.test_client() as c:
        for customer_id in range(0, 0x10000, 0x7FE):
            with app.test_request_context():
                url = url_for(encoder, customer_id=customer_id)
            rv = c.get(url)
            assert 200 == rv.status_code
            assert rv.data.startswith(B(encoder))
            assert rv.data.endswith(B(obs.to_url(customer_id)))


@pytest.mark.parametrize("encoder", FILTERS)
def test_jinja2_filter(encoder):
    obs = Obscure(SALT)

    def conv(x):
        return str(getattr(obs, TRANSLATE[encoder])(x))

    jinja2_filter = make_jinja_filter_for(encoder)
    env = Environment()
    env.filters[encoder] = jinja2_filter
    tmpl = env.from_string("{{ x|%s }}" % encoder)

    for x in range(0xFE, 0x1000, 0x1FE):
        rv = tmpl.render(x=x)
        assert rv == conv(x)

    with pytest.raises(TypeError):
        tmpl.render(x="silly string")



"""Obscure sequential IDs in URL variables and templates.

Impliments routing converters and fiters in Flask to obscure sequential
integer IDs.  This is base on the 'Obscure' python module.

Once installed, the following converters and filters are available:
    num, hex, b32, b64, and tame
"""

from werkzeug.routing import BaseConverter, IntegerConverter
from obscure import Obscure as _mod_Obscure, _base32_custom as _tame_alphabet


class Obscure(_mod_Obscure):
    """Obscure interger IDs in URLs.
    A ``salt`` value is needed.  You can provide it when initializing
    the app or from the flask configuration under the parameter
    ``OBSCURE_SALT``.
    """

    def __init__(self, app=None, salt=None):
        """Add converters and filters to a :class:`Flask` instance.

        Args:
          app: a :class:`flask:Flask` instance or None
          salt (integer): random 32-bit integer for uniqueness
        """
        self.salt = salt
        if app is not None:
            self.init_app(app, self.salt)

    def init_app(self, app, salt=None):
        """Add converters and filters to a :class:`Flask` instance.

        Args:
          app: a :class:`flask:Flask` instance
          salt (integer): random 32-bit integer for uniqueness

        Raises:
            KeyError: ``OBSCURE_SALT`` must be in the
             :class:`flask.Config` if it is not given as a parameter.
        """
        salt = salt or self.salt or int(app.config["OBSCURE_SALT"])
        _mod_Obscure.__init__(self, salt)

        for converter_name, base in converters.items():
            class_name = "Obscure" + base.__name__
            class_ = type(class_name, (base,), {"obscure": self})
            app.url_map.converters[converter_name] = class_

            def filter_(x, c=class_, s=salt):
                return c(s).to_url(x)

            app.add_template_filter(filter_, converter_name)


class Num(IntegerConverter):
    """Obscure interger ID with salted value and format as
    an alternative, non-sequential number.

    Rule('/customer/<num:customer_id>')
    """

    def __init__(self, map):
        IntegerConverter.__init__(self, map, max=0xFFFFFFFF)

    def to_python(self, value):
        """Restores original number.

        Args:
          value (number string): obscured, non-sequential number

        Returns:
            integer: the original number

        See Also:
            to_url
        """
        value = IntegerConverter.to_python(self, int(value))
        return self.obscure.transform(value)

    def to_url(self, value):
        """Convert value to alternate, non-sequential integer format.

        Args:
          value (integer): number to obscure

        Returns:
          string: an obscured, non-sequential number

        See Also:
          to_python
        """
        """
        :param value: integer to convert
        :returns: integer as a string
        """
        return str(self.obscure.transform(value))


class Hex(BaseConverter):
    """Obscure numerical ID and format as hex.

    Rule('/customer/<hex:customer_id>')
    """

    weight = 50
    regex = "[abcdef0123456789]{8}"

    def to_python(self, value):
        """Restores original number.

        Args:
          value: 8 digit hexadecimal string

        Returns:
          integer: original integer

        See Also:
          to_url
        """
        """
        :param value: 8 digit hexadecimal format string
        :returns: integer
        """
        return self.obscure.decode_hex(value)

    def to_url(self, value):
        """Convert value to hexadecimal format.

        :param value: integer to convert
        :returns: string in hexadecimal format
        """
        return self.obscure.encode_hex(value)


class Base32(BaseConverter):
    """Obscure numerical ID and format as base32.

    Rule('/customer/<b32:customer_id>')
    """

    weight = 50
    regex = "[A-Z2-7]{7}"

    def to_python(self, value):
        """Restores original number.

        :param value: 7 digit base32 format string
        :returns: integer
        """

        return self.obscure.decode_base32(str(value))

    def to_url(self, value):
        """Convert value to Base32 format.

        :param value: integer to convert
        :returns: string in Base32 format
        """
        return self.obscure.encode_base32(value)


class Base64(BaseConverter):
    """Obscure numerical ID and format as url-safe base64.

    Rule('/customer/<b64:customer_id>')
    """

    weight = 50
    regex = "[-_A-Za-z0-9]{6}"

    def to_python(self, value):
        """Restores original number.

        :param value: alternate Base64 format string
        :returns: integer
        """
        return self.obscure.decode_base64(str(value))

    def to_url(self, value):
        """Convert value to Base64 format.

        :param value: integer to convert
        :returns: string in Base64 format
        """
        return self.obscure.encode_base64(value)


class Tame(BaseConverter):
    """Obscure numerical ID and format as a custom base32
    with the vowels 'I', 'O', and 'U' removed to eliminate common
    offensive words.

    Rule('/customer/<tame:customer_id>')
    """

    weight = 50
    regex = "[%s]{7}" % _tame_alphabet

    def to_python(self, value):
        """Restores original number.

        :param value: alternate Base32 format string
        :returns: integer
        """
        return self.obscure.decode_tame(str(value))

    def to_url(self, value):
        """Convert value to alternate Base32 format.

        :param value: integer to convert
        :returns: alternate Base32 format
        """
        return self.obscure.encode_tame(value)


converters = {"num": Num, "hex": Hex, "tame": Tame, "b32": Base32, "b64": Base64}



import sys
import itertools
import pytest
from flask import Flask, request, url_for, render_template_string
from jinja2 import Environment
from helper import FILTERS, METHODS, make_filters


def B(s):
    if sys.version_info.major == 2:
        return s
    return bytes(s, 'ascii')


def create_app(method):
    my_app = Flask('test_filter')

    filters = make_filters(FILTERS, method)
    for name, function in filters.items():
        my_app.add_template_filter(function, name)

    @my_app.route('/my_hex/<int:number>', endpoint='my_hex')
    @my_app.route('/my_bin/<int:number>', endpoint='my_bin')
    @my_app.route('/my_oct/<int:number>', endpoint='my_oct')
    def use_filter(number):
        filter_ = request.url_rule.endpoint
        template = '{{ filter_ }} {{ number|%s }}' % filter_
        pytest.set_trace()
        return render_template_string(template, **locals())
    return my_app


def test_flask_oct():
    app = create_app('LAMBDA')
    with app.test_client() as go:
        with app.test_request_context():
            url = url_for('my_oct', number=254)
        rv = go.get(url)
        assert 200 == rv.status_code
        assert rv.data.startswith(B('my_oct'))
        assert rv.data.endswith(B('0o376'))


@pytest.mark.parametrize('method,filter_',
                         itertools.product(METHODS, FILTERS))
def test_flask(method, filter_):
    NUM = 254
    app = create_app(method)
    with app.test_client() as go:
        with app.test_request_context():
            url = url_for(filter_, number=NUM)
        rv = go.get(url)
        assert 200 == rv.status_code
        func = __builtins__[filter_[3:]]
        assert rv.data.startswith(B(filter_))
        assert rv.data.endswith(B(func(NUM)))


@pytest.mark.parametrize('method,filter_',
                         itertools.product(METHODS, FILTERS))
def test_jinja2(method, filter_):
    filters = make_filters(FILTERS, method)
    env = Environment()
    env.filters.update(filters)
    for number in range(0, 0x10000, 0xfe):
        template = env.from_string("{{ filter_ }} {{ number|%s }}" % filter_)
        rv = template.render(**locals())
        assert rv.startswith(filter_)
        assert rv.endswith(filters[filter_](number))



import pytest
import json
from flask import Flask, url_for, request, jsonify
from flask_obscure import Obscure, converters as _converters

customer_names = ["Adam", "Betty", "Carlos"]
converters = tuple(_converters.keys())
obs = Obscure()


@pytest.fixture(scope="function")
def app():
    _app = Flask(__name__)
    _app.config["OBSCURE_SALT"] = 0x1234  # Use a proper config
    obs.init_app(_app)

    def get_all():
        endpoint = request.url_rule.endpoint.split("-")[-1]
        return jsonify({"data": [c.to_dict(endpoint=endpoint) for c in customers]})

    # Add a route endpoint for each converter
    for conv in reversed(converters):
        get_all = _app.route("/customers/%s" % conv, endpoint="cust-%s" % conv)(get_all)

    def get_cust(customer_id):
        customer = customers[customer_id].to_dict(False)
        rv = {"legal": "FOR INTERNAL USE ONLY! UNOBSCURED"}
        rv.update(customer)
        return jsonify(rv)

    # Add a route endpoint for each converter
    for conv in converters:
        get_cust = _app.route("/{0}/<{0}:customer_id>".format(conv), endpoint=conv)(
            get_cust
        )
    return _app


class Generic(object):
    """This is taking the place of any database object.
    The `to_dict` looks ugly but since you will be using a single
    converter in your code, the bulk of the code goes away.
    """

    def __init__(self, **kwargs):
        self.fields = kwargs
        self.converters = {
            "num": obs.transform,
            "hex": obs.encode_hex,
            "b32": obs.encode_base32,
            "b64": obs.encode_base64,
            "tame": obs.encode_tame,
        }

    def to_dict(self, obscure=True, endpoint=None):
        fields = self.fields.copy()
        url = None
        if obscure is True:
            for k, v in fields.items():
                if k.lower().endswith("_id"):
                    if endpoint is not None:
                        url = url_for(endpoint, **{k: v})
                    v = self.converters[endpoint](v)
                    fields[k] = v
        if url is not None:
            fields["url"] = url
        return fields


customers = [Generic(name=_, customer_id=idx) for idx, _ in enumerate(customer_names)]


def rv2json(data):
    return json.loads(data.decode("ascii"))


@pytest.mark.parametrize("converter", converters)
def test_obscured_json_output(app, converter):
    """Verify the URLs on each customer."""
    with app.test_client() as c:
        with app.test_request_context():
            url = url_for("cust-" + converter)
        rv = c.get(url)
        assert rv.status_code == 200
        assert request.endpoint.endswith(converter)
        data = rv2json(rv.data)["data"]
        assert customer_names == [_["name"] for _ in data]

        for idx, url in enumerate([_["url"] for _ in data]):
            assert url[1:].startswith(converter)
            rv2 = c.get(url)
            assert rv.status_code == 200
            assert request.endpoint == converter
            cust_data = rv2json(rv2.data)
            assert "legal" in cust_data
            assert cust_data["name"] == data[idx]["name"]
            assert cust_data["customer_id"] == idx


@pytest.mark.parametrize(
    "url",
    [
        "/num/",  # missing number
        "/num/ABCDEF",  # bad no letters
        "/num/%s" % str(int(0xFFFFFFFF) + 1),  # too large
        "/hex/abcd",  # to short
        "/hex/AABBCCDD",  # bad CAPS
        "/hex/aabbccdg",  # bad 'g'
        "/b32/ANUIT01",  # bad '01'
        "/b32/anuit23",  # bad lower case
        "/b32/WRONGLEN",  # bad length
        "/b64/some+",  # bad '+'
        "/b64/len",  # bad length
        "/tame/CURSEWD",  # bad 'U'
        "/tame/IGNORES",  # bad 'I'
    ],
)
def test_bad_converter_regex(app, url):
    with app.test_client() as c:
        rv = c.get(url)
        assert rv.status_code == 404


@pytest.mark.parametrize(
    "url",
    ["/num/1234", "/hex/a1b2c3d4", "/b32/TESTING", "/b64/some-_", "/tame/SVCCESS",],
)
def test_bad_customer_id(app, url):
    """The URL is has proper coding but there is no such
    customer, which generates a IndexError and a 500 error.
    """
    with app.test_client() as c:
        rv = c.get(url)
        assert rv.status_code == 500



from pkg_resources import get_distribution, DistributionNotFound
from .obscure import Obscure, converters, Num, Hex, Base32, Base64, Tame

__all__ = [
    "__version__",
    "Obscure",
    "converters",
    "Num",
    "Hex",
    "Base32",
    "Base64",
    "Tame",
]

try:
    __version__ = get_distribution(__name__).version
except DistributionNotFound:
    pass  # package is not installed
